@@ -1,10 +1,13 @@
 from sympy import I
 from sympy.physics.paulialgebra import Pauli
+from sympy.utilities.pytest import XFAIL
+
+sigma1 = Pauli(1)
+sigma2 = Pauli(2)
+sigma3 = Pauli(3)
 
 def test_Pauli():
-    sigma1=Pauli(1)
-    sigma2=Pauli(2)
-    sigma3=Pauli(3)
+    from sympy.physics.paulialgebra import evaluate_pauli_product
 
     assert sigma1 == sigma1
     assert sigma1 != sigma2
@@ -17,6 +20,8 @@ def test_Pauli():
     assert sigma2*sigma2 == 1
     assert sigma3*sigma3 == 1
 
+    assert evaluate_pauli_product(I*sigma2*sigma3) == -sigma1
+
     assert sigma1**0 == 1
     assert sigma1**1 == sigma1
     assert sigma1**2 == 1
@@ -25,6 +30,8 @@ def test_Pauli():
 
     assert sigma3**2 == 1
 
-
     assert sigma1*2*sigma1 == 2
-    #assert sigma1*sigma3*sigma1 == -sigma3     XXX should work
+
+@XFAIL
+def test_Pauli_should_work():
+    assert sigma1*sigma3*sigma1 == -sigma3
