@@ -1,7 +1,13 @@
-from sympy.core import Basic, C
-from sympy.core.compatibility import is_sequence, iterable #logically, they belong here
-
+from collections import defaultdict
 import random
+from operator import gt
+
+from sympy.core import Basic, C
+from sympy.core.compatibility import is_sequence, iterable # logical location
+from sympy.core.compatibility import as_int, quick_sort,\
+    product as cartes, combinations, combinations_with_replacement
+from sympy.utilities.misc import default_sort_key
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 def flatten(iterable, levels=None, cls=None):
     """
@@ -48,7 +54,7 @@ def flatten(iterable, levels=None, cls=None):
             raise ValueError("expected non-negative number of levels, got %s" % levels)
 
     if cls is None:
-        reducible = lambda x: hasattr(x, "__iter__") and not isinstance(x, basestring)
+        reducible = lambda x: is_sequence(x, set)
     else:
         reducible = lambda x: isinstance(x, cls)
 
@@ -64,6 +70,69 @@ def flatten(iterable, levels=None, cls=None):
 
     return result
 
+def unflatten(iter, n=2):
+    """Group ``iter`` into tuples of length ``n``. Raise an error if
+    the length of ``iter`` is not a multiple of ``n``.
+    """
+    if n < 1 or len(iter) % n:
+        raise ValueError('iter length is not a multiple of %i' % n)
+    return zip(*(iter[i::n] for i in xrange(n)))
+
+def reshape(seq, how):
+    """Reshape the sequence according to the template in ``how``.
+
+    Examples
+    ========
+
+    >>> from sympy.utilities import reshape
+    >>> seq = range(1, 9)
+
+    >>> reshape(seq, [4]) # lists of 4
+    [[1, 2, 3, 4], [5, 6, 7, 8]]
+
+    >>> reshape(seq, (4,)) # tuples of 4
+    [(1, 2, 3, 4), (5, 6, 7, 8)]
+
+    >>> reshape(seq, (2, 2)) # tuples of 4
+    [(1, 2, 3, 4), (5, 6, 7, 8)]
+
+    >>> reshape(seq, (2, [2])) # (i, i, [i, i])
+    [(1, 2, [3, 4]), (5, 6, [7, 8])]
+
+    >>> reshape(seq, ((2,), [2])) # etc....
+    [((1, 2), [3, 4]), ((5, 6), [7, 8])]
+
+    >>> reshape(seq, (1, [2], 1))
+    [(1, [2, 3], 4), (5, [6, 7], 8)]
+
+    >>> reshape(tuple(seq), ([[1], 1, (2,)],))
+    (([[1], 2, (3, 4)],), ([[5], 6, (7, 8)],))
+
+    >>> reshape(tuple(seq), ([1], 1, (2,)))
+    (([1], 2, (3, 4)), ([5], 6, (7, 8)))
+    """
+    m = sum(flatten(how))
+    n, rem = divmod(len(seq), m)
+    if m < 0 or rem:
+        raise ValueError('template must sum to positive number '
+        'that divides the length of the sequence')
+    i = 0
+    container = type(how)
+    rv = [None]*n
+    for k in range(len(rv)):
+        rv[k] = []
+        for hi in how:
+            if type(hi) is int:
+                rv[k].extend(seq[i: i + hi])
+                i += hi
+            else:
+                n = sum(flatten(hi))
+                fg = type(hi)
+                rv[k].append(fg(reshape(seq[i: i + n], hi))[0])
+                i += n
+        rv[k] = container(rv[k])
+    return type(seq)(rv)
+
 def group(container, multiple=True):
     """
     Splits a container into a list of lists of equal, adjacent elements.
@@ -99,7 +168,7 @@ def group(container, multiple=True):
 
     return groups
 
-def postorder_traversal(node):
+def postorder_traversal(node, key=None):
     """
     Do a postorder traversal of a tree.
 
@@ -108,105 +177,48 @@ def postorder_traversal(node):
     a node's children's postorder traversal before yielding the node itself.
 
     Parameters
-    ----------
+    ==========
     node : sympy expression
         The expression to traverse.
+    key : (default None) sort key
+        The key used to sort args of Basic objects. When None, args of Basic
+        objects are processed in arbitrary order.
 
-    Yields
-    ------
+    Returns
+    =======
     subtree : sympy expression
         All of the subtrees in the tree.
 
     Examples
-    --------
-    >>> from sympy import symbols
+    ========
+    >>> from sympy import symbols, default_sort_key
     >>> from sympy.utilities.iterables import postorder_traversal
-    >>> from sympy.abc import x, y, z
-    >>> set(postorder_traversal((x+y)*z)) == set([z, y, x, x + y, z*(x + y)])
-    True
+    >>> from sympy.abc import w, x, y, z
+
+    The nodes are returned in the order that they are encountered unless key
+    is given.
+
+    >>> list(postorder_traversal(w + (x + y)*z)) # doctest: +SKIP
+    [z, y, x, x + y, z*(x + y), w, w + z*(x + y)]
+    >>> list(postorder_traversal(w + (x + y)*z, key=default_sort_key))
+    [w, z, x, y, x + y, z*(x + y), w + z*(x + y)]
+
 
     """
     if isinstance(node, Basic):
-        for arg in node.args:
-            for subtree in postorder_traversal(arg):
+        args = node.args
+        if key:
+            args = list(args)
+            args.sort(key=key)
+        for arg in args:
+            for subtree in postorder_traversal(arg, key):
                 yield subtree
     elif iterable(node):
         for item in node:
-            for subtree in postorder_traversal(item):
+            for subtree in postorder_traversal(item, key):
                 yield subtree
     yield node
 
-class preorder_traversal(object):
-    """
-    Do a pre-order traversal of a tree.
-
-    This iterator recursively yields nodes that it has visited in a pre-order
-    fashion. That is, it yields the current node then descends through the tree
-    breadth-first to yield all of a node's children's pre-order traversal.
-
-    Parameters
-    ----------
-    node : sympy expression
-        The expression to traverse.
-
-    Yields
-    ------
-    subtree : sympy expression
-        All of the subtrees in the tree.
-
-    Examples
-    --------
-    >>> from sympy import symbols
-    >>> from sympy.utilities.iterables import preorder_traversal
-    >>> from sympy.abc import x, y, z
-    >>> set(preorder_traversal((x+y)*z)) == set([z, x + y, z*(x + y), x, y])
-    True
-
-    """
-    def __init__(self, node):
-        self._skip_flag = False
-        self._pt = self._preorder_traversal(node)
-
-    def _preorder_traversal(self, node):
-        yield node
-        if self._skip_flag:
-            self._skip_flag = False
-            return
-        if isinstance(node, Basic):
-            for arg in node.args:
-                for subtree in self._preorder_traversal(arg):
-                    yield subtree
-        elif iterable(node):
-            for item in node:
-                for subtree in self._preorder_traversal(item):
-                    yield subtree
-
-    def skip(self):
-        """
-        Skip yielding current node's (last yielded node's) subtrees.
-
-        Examples
-        --------
-        >>> from sympy import symbols
-        >>> from sympy.utilities.iterables import preorder_traversal
-        >>> from sympy.abc import x, y, z
-        >>> pt = preorder_traversal((x+y*z)*z)
-        >>> for i in pt:
-        ...     print i
-        ...     if i == x+y*z:
-        ...             pt.skip()
-        z*(x + y*z)
-        z
-        x + y*z
-        """
-        self._skip_flag = True
-
-    def next(self):
-        return self._pt.next()
-
-    def __iter__(self):
-        return self
-
 def interactive_traversal(expr):
     """Traverse a tree asking a user which branch to choose. """
     from sympy.printing import pprint
@@ -297,75 +309,54 @@ def interactive_traversal(expr):
 
     return _interactive_traversal(expr, 0)
 
-def cartes(*seqs):
-    """Return Cartesian product (combinations) of items from iterable
-    sequences, seqs, as a generator.
-
-    Examples::
-    >>> from sympy import Add, Mul
-    >>> from sympy.abc import x, y
-    >>> from sympy.utilities.iterables import cartes
-    >>> do=list(cartes([Mul, Add], [x, y], [2]))
-    >>> for di in do:
-    ...     print di[0](*di[1:])
-    ...
-    2*x
-    2*y
-    x + 2
-    y + 2
-    >>>
-
-    >>> list(cartes([1, 2], [3, 4, 5]))
-    [[1, 3], [1, 4], [1, 5], [2, 3], [2, 4], [2, 5]]
-    """
-
-    if not seqs:
-        yield []
-    else:
-        for item in seqs[0]:
-            for subitem in cartes(*seqs[1:]):
-                yield [item] + subitem
-
 def variations(seq, n, repetition=False):
-    """Returns a generator of the variations (size n) of the list `seq` (size N).
-    `repetition` controls whether items in seq can appear more than once;
+    """Returns a generator of the n-sized variations of ``seq`` (size N).
+    ``repetition`` controls whether items in ``seq`` can appear more than once;
 
-    Examples:
+    Examples
+    ========
 
     variations(seq, n) will return N! / (N - n)! permutations without
     repetition of seq's elements:
+
         >>> from sympy.utilities.iterables import variations
         >>> list(variations([1, 2], 2))
-        [[1, 2], [2, 1]]
+        [(1, 2), (2, 1)]
 
     variations(seq, n, True) will return the N**n permutations obtained
     by allowing repetition of elements:
+
         >>> list(variations([1, 2], 2, repetition=True))
-        [[1, 1], [1, 2], [2, 1], [2, 2]]
+        [(1, 1), (1, 2), (2, 1), (2, 2)]
 
     If you ask for more items than are in the set you get the empty set unless
     you allow repetitions:
+
         >>> list(variations([0, 1], 3, repetition=False))
         []
         >>> list(variations([0, 1], 3, repetition=True))[:4]
-        [[0, 0, 0], [0, 0, 1], [0, 1, 0], [0, 1, 1]]
+        [(0, 0, 0), (0, 0, 1), (0, 1, 0), (0, 1, 1)]
 
+    See Also
+    ========
 
-    Reference:
-        http://code.activestate.com/recipes/190465/
+    sympy.core.compatibility.permutations
     """
+    from sympy.core.compatibility import permutations
 
-    if n == 0:
-        yield []
+    if not repetition:
+        seq = tuple(seq)
+        if len(seq) < n:
+            return
+        for i in permutations(seq, n):
+            yield i
     else:
-        if not repetition:
-            for i in xrange(len(seq)):
-                for cc in variations(seq[:i] + seq[i + 1:], n - 1, False):
-                    yield [seq[i]] + cc
+        if n == 0:
+            yield ()
         else:
             for i in xrange(len(seq)):
                 for cc in variations(seq, n - 1, True):
-                    yield [seq[i]] + cc
+                    yield (seq[i],) + cc
 
 def subsets(seq, k=None, repetition=False):
     """Generates all k-subsets (combinations) from an n-element set, seq.
@@ -375,62 +366,48 @@ def subsets(seq, k=None, repetition=False):
        whereas there are 2**n subsets all together. If k is None then all
        2**n subsets will be returned from shortest to longest.
 
-       Examples:
-           >>> from sympy.utilities.iterables import subsets
+       Examples
+       ========
+       >>> from sympy.utilities.iterables import subsets
 
        subsets(seq, k) will return the n!/k!/(n - k)! k-subsets (combinations)
        without repetition, i.e. once an item has been removed, it can no
        longer be "taken":
+
            >>> list(subsets([1, 2], 2))
-           [[1, 2]]
+           [(1, 2)]
            >>> list(subsets([1, 2]))
-           [[], [1], [2], [1, 2]]
+           [(), (1,), (2,), (1, 2)]
            >>> list(subsets([1, 2, 3], 2))
-           [[1, 2], [1, 3], [2, 3]]
+           [(1, 2), (1, 3), (2, 3)]
 
 
        subsets(seq, k, repetition=True) will return the (n - 1 + k)!/k!/(n - 1)!
        combinations *with* repetition:
+
            >>> list(subsets([1, 2], 2, repetition=True))
-           [[1, 1], [1, 2], [2, 2]]
+           [(1, 1), (1, 2), (2, 2)]
 
        If you ask for more items than are in the set you get the empty set unless
        you allow repetitions:
+
            >>> list(subsets([0, 1], 3, repetition=False))
            []
            >>> list(subsets([0, 1], 3, repetition=True))
-           [[0, 0, 0], [0, 0, 1], [0, 1, 1], [1, 1, 1]]
-       """
+           [(0, 0, 0), (0, 0, 1), (0, 1, 1), (1, 1, 1)]
 
-    if type(seq) is not list:
-        seq = list(seq)
-    if k == 0:
-        yield []
-    elif k is None:
-        yield []
-        for k in range(1, len(seq) + 1):
-            for s in subsets(seq, k, repetition=repetition):
-                yield list(s)
+       """
+    if k is None:
+        for k in range(len(seq) + 1):
+            for i in subsets(seq, k, repetition):
+                yield i
     else:
         if not repetition:
-            for i in xrange(len(seq)):
-                for cc in subsets(seq[i + 1:], k - 1, False):
-                    yield [seq[i]] + cc
+            for i in combinations(seq, k):
+                yield i
         else:
-            nmax = len(seq) - 1
-            indices = [0] * k
-            yield seq[:1] * k
-            while 1:
-                indices[-1] += 1
-                if indices[-1] > nmax:
-                    #find first digit that can be incremented
-                    for j in range(-2, -k - 1, -1):
-                        if indices[j] < nmax:
-                            indices[j:] = [indices[j] + 1] * -j
-                            break # increment and copy to the right
-                    else:
-                        break # we didn't for-break so we are done
-                yield [seq[li] for li in indices]
+            for i in combinations_with_replacement(seq, k):
+                yield i
 
 def numbered_symbols(prefix='x', cls=None, start=0, *args, **assumptions):
     """
@@ -449,19 +426,24 @@ def numbered_symbols(prefix='x', cls=None, start=0, *args, **assumptions):
     start : int, optional
         The start number.  By default, it is 0.
 
-    Yields
-    ------
+    Returns
+    -------
     sym : Symbol
         The subscripted symbols.
     """
-    if cls is None:
-        if 'dummy' in assumptions and assumptions.pop('dummy'):
-            import warnings
-            warnings.warn("\nuse cls=Dummy to create dummy symbols",
-                          DeprecationWarning)
+    if 'dummy' in assumptions:
+        SymPyDeprecationWarning(
+            feature="'dummy=True' to create numbered Dummy symbols",
+            useinstead="cls=Dummy",
+            issue=3378, deprecated_since_version="0.7.0",
+        ).warn()
+        if assumptions.pop('dummy'):
             cls = C.Dummy
-        else:
-            cls = C.Symbol
+
+    if cls is None:
+        # We can't just make the default cls=C.Symbol because it isn't
+        # imported yet.
+        cls = C.Symbol
 
     while True:
         name = '%s%s' % (prefix, start)
@@ -471,27 +453,35 @@ def numbered_symbols(prefix='x', cls=None, start=0, *args, **assumptions):
 def capture(func):
     """Return the printed output of func().
 
-    `func` should be an argumentless function that produces output with
+    `func` should be a function without arguments that produces output with
     print statements.
 
     >>> from sympy.utilities.iterables import capture
+    >>> from sympy import pprint
+    >>> from sympy.abc import x
     >>> def foo():
     ...     print 'hello world!'
     ...
     >>> 'hello' in capture(foo) # foo, not foo()
     True
+    >>> capture(lambda: pprint(2/x))
+    '2\\n-\\nx\\n'
+
     """
     import StringIO
     import sys
 
     stdout = sys.stdout
     sys.stdout = file = StringIO.StringIO()
-    func()
-    sys.stdout = stdout
+    try:
+        func()
+    finally:
+        sys.stdout = stdout
     return file.getvalue()
 
 def sift(expr, keyfunc):
-    """Sift the arguments of expr into a dictionary according to keyfunc.
+    """
+    Sift the arguments of expr into a dictionary according to keyfunc.
 
     INPUT: expr may be an expression or iterable; if it is an expr then
     it is converted to a list of expr's args or [expr] if there are no args.
@@ -499,7 +489,11 @@ def sift(expr, keyfunc):
     OUTPUT: each element in expr is stored in a list keyed to the value
     of keyfunc for the element.
 
-    EXAMPLES:
+    Note that for a SymPy expression, the order of the elements in the lists
+    is dependent on the order in .args, which can be arbitrary.
+
+    Examples
+    ========
 
     >>> from sympy.utilities import sift
     >>> from sympy.abc import x, y
@@ -508,27 +502,33 @@ def sift(expr, keyfunc):
     >>> sift(range(5), lambda x: x%2)
     {0: [0, 2, 4], 1: [1, 3]}
 
-    It is possible that some keys are not present, in which case you should
-    used dict's .get() method:
+    sift() returns a defaultdict() object, so any key that has no matches will
+    give [].
 
-    >>> sift(x+y, lambda x: x.is_commutative)
-    {True: [y, x]}
-    >>> _.get(False, [])
+    >>> sift(x, lambda x: x.is_commutative)
+    {True: [x]}
+    >>> _[False]
     []
 
     Sometimes you won't know how many keys you will get:
-    >>> sift(sqrt(x) + x**2 + exp(x) + (y**x)**2,
-    ... lambda x: x.as_base_exp()[0])
-    {E: [exp(x)], x: [x**(1/2), x**2], y: [y**(2*x)]}
-    >>> _.keys()
-    [E, x, y]
 
+    >>> sift(sqrt(x) + exp(x) + (y**x)**2,
+    ... lambda x: x.as_base_exp()[0])
+    {E: [exp(x)], x: [sqrt(x)], y: [y**(2*x)]}
+
+    If you need to sort the sifted items it might be better to use
+    the lazyDSU_sort which can economically apply multiple sort keys
+    to a squence while sorting.
+
+    See Also
+    ========
+    lazyDSU_sort
     """
-    d = {}
+    d = defaultdict(list)
     if hasattr(expr, 'args'):
         expr = expr.args or [expr]
     for e in expr:
-        d.setdefault(keyfunc(e), []).append(e)
+        d[keyfunc(e)].append(e)
     return d
 
 def take(iter, n):
@@ -544,12 +544,66 @@ def dict_merge(*dicts):
 
     return merged
 
+def common_prefix(*seqs):
+    """Return the subsequence that is a common start of sequences in ``seqs``.
+
+    >>> from sympy.utilities.iterables import common_prefix
+    >>> common_prefix(range(3))
+    [0, 1, 2]
+    >>> common_prefix(range(3), range(4))
+    [0, 1, 2]
+    >>> common_prefix([1, 2, 3], [1, 2, 5])
+    [1, 2]
+    >>> common_prefix([1, 2, 3], [1, 3, 5])
+    [1]
+    """
+    if any(not s for s in seqs):
+        return []
+    elif len(seqs) == 1:
+        return seqs[0]
+    i = 0
+    for i in range(min(len(s) for s in seqs)):
+        if not all(seqs[j][i] == seqs[0][i] for j in xrange(len(seqs))):
+            break
+    else:
+        i += 1
+    return seqs[0][:i]
+
+def common_suffix(*seqs):
+    """Return the subsequence that is a common ending of sequences in ``seqs``.
+
+    >>> from sympy.utilities.iterables import common_suffix
+    >>> common_suffix(range(3))
+    [0, 1, 2]
+    >>> common_suffix(range(3), range(4))
+    []
+    >>> common_suffix([1, 2, 3], [9, 2, 3])
+    [2, 3]
+    >>> common_suffix([1, 2, 3], [9, 7, 3])
+    [3]
+    """
+
+    if any(not s for s in seqs):
+        return []
+    elif len(seqs) == 1:
+        return seqs[0]
+    i = 0
+    for i in range(-1, -min(len(s) for s in seqs) - 1, -1):
+        if not all(seqs[j][i] == seqs[0][i] for j in xrange(len(seqs))):
+            break
+    else:
+        i -= 1
+    if i == -1:
+        return []
+    else:
+        return seqs[0][i + 1:]
+
 def prefixes(seq):
     """
     Generate all prefixes of a sequence.
 
-    Example
-    =======
+    Examples
+    ========
 
     >>> from sympy.utilities.iterables import prefixes
 
@@ -566,8 +620,8 @@ def postfixes(seq):
     """
     Generate all postfixes of a sequence.
 
-    Example
-    =======
+    Examples
+    ========
 
     >>> from sympy.utilities.iterables import postfixes
 
@@ -595,7 +649,8 @@ def topological_sort(graph, key=None):
         (e.g. lexicographic) ordering is used (in this case the base type
         must implement ordering relations).
 
-    **Examples**
+    Examples
+    ========
 
     Consider a graph::
 
@@ -693,7 +748,9 @@ def rotate_left(x, y):
     Left rotates a list x by the number of steps specified
     in y.
 
-    Examples:
+    Examples
+    ========
+
     >>> from sympy.utilities.iterables import rotate_left
     >>> a = [0, 1, 2]
     >>> rotate_left(a, 1)
@@ -709,7 +766,9 @@ def rotate_right(x, y):
     Left rotates a list x by the number of steps specified
     in y.
 
-    Examples:
+    Examples
+    ========
+
     >>> from sympy.utilities.iterables import rotate_right
     >>> a = [0, 1, 2]
     >>> rotate_right(a, 1)
@@ -729,7 +788,9 @@ def multiset_partitions(multiset, m):
     m-partitions, that is, all partitions having exactly size m
     using auxiliary arrays as described in the book.
 
-    Examples:
+    Examples
+    ========
+
     >>> from sympy.utilities.iterables import multiset_partitions
     >>> list(multiset_partitions([1,2,3,4], 2))
     [[[1, 2, 3], [4]], [[1, 3], [2, 4]], [[1], [2, 3, 4]], [[1, 2], \
@@ -738,6 +799,11 @@ def multiset_partitions(multiset, m):
     [[[1, 2, 3, 4]]]
     >>> list(multiset_partitions([1,2,3,4], 4))
     [[[1], [2], [3], [4]]]
+
+    See Also
+    ========
+    sympy.combinatorics.partitions.Partition
+    sympy.combinatorics.partitions.IntegerPartition
     """
     cache = {}
 
@@ -753,7 +819,7 @@ def multiset_partitions(multiset, m):
     def f(m_arr, n_arr, sigma, n, a):
         if m_arr <= 2:
             v = visit(n, a)
-            if not v is None:
+            if v is not None:
                 yield v
         else:
             for v in f(m_arr - 1, n_arr - 1, (m_arr + sigma) % 2, n, a):
@@ -761,12 +827,12 @@ def multiset_partitions(multiset, m):
         if n_arr == m_arr + 1:
             a[m_arr] = m_arr - 1
             v = visit(n, a)
-            if not v is None:
+            if v is not None:
                 yield v
             while a[n_arr] > 0:
                 a[n_arr] = a[n_arr] - 1
                 v = visit(n, a)
-                if not v is None:
+                if v is not None:
                     yield v
         elif n_arr > m_arr + 1:
             if (m_arr + sigma) % 2 == 1:
@@ -787,16 +853,16 @@ def multiset_partitions(multiset, m):
     def b(m_arr, n_arr, sigma, n, a):
         if n_arr == m_arr + 1:
             v = visit(n, a)
-            if not v is None:
+            if v is not None:
                 yield v
             while a[n_arr] < m_arr - 1:
                 a[n_arr] = a[n_arr] + 1
                 v = visit(n, a)
-                if not v is None:
+                if v is not None:
                     yield v
             a[m_arr] = 0
             v = visit(n, a)
-            if not v is None:
+            if v is not None:
                 yield v
         elif n_arr > m_arr + 1:
             func = [f, b][(a[n_arr] + sigma) % 2]
@@ -815,7 +881,7 @@ def multiset_partitions(multiset, m):
                 a[m_arr] = 0
         if m_arr <= 2:
             v = visit(n, a)
-            if not v is None:
+            if v is not None:
                 yield v
         else:
             for v in b(m_arr - 1, n_arr - 1, (m_arr + sigma) % 2, n, a):
@@ -877,10 +943,16 @@ def partitions(n, m=None, k=None):
     Reference:
         modified from Tim Peter's version to allow for k and m values:
         code.activestate.com/recipes/218332-generator-for-integer-partitions/
-    """
 
+    See Also
+    ========
+    sympy.combinatorics.partitions.Partition
+    sympy.combinatorics.partitions.IntegerPartition
+    """
     if n < 0:
         raise ValueError("n must be >= 0")
+    if m == 0:
+        raise ValueError("m must be > 0")
     m = min(m or n, n)
     if m < 1:
         raise ValueError("maximum numbers in partition, m, must be > 0")
@@ -891,6 +963,7 @@ def partitions(n, m=None, k=None):
     if m*k < n:
         return
 
+    n, m, k = as_int(n), as_int(m), as_int(k)
     q, r = divmod(n, k)
     ms = {k: q}
     keys = [k]  # ms.keys(), from largest to smallest
@@ -948,7 +1021,9 @@ def binary_partitions(n):
 
     Reference: TAOCP 4, section 7.2.1.5, problem 64
 
-    Examples:
+    Examples
+    ========
+
     >>> from sympy.utilities.iterables import binary_partitions
     >>> for i in binary_partitions(5):
     ...     print i
@@ -989,16 +1064,58 @@ def binary_partitions(n):
                 x >>= 1
     yield [1]*n
 
+def has_dups(seq):
+    """Return True if there are any duplicate elements in ``seq``.
+
+    Examples
+    ========
+    >>> from sympy.utilities.iterables import has_dups
+    >>> from sympy import Dict, Set
+
+    >>> has_dups((1, 2, 1))
+    True
+    >>> has_dups(range(3))
+    False
+    >>> all(has_dups(c) is False for c in (set(), Set(), dict(), Dict()))
+    True
+    """
+    if isinstance(seq, (dict, set, C.Dict, C.Set)):
+        return False
+    uniq = set()
+    return any(True for s in seq if s in uniq or uniq.add(s))
+
+def has_variety(seq):
+    """Return True if there are any different elements in ``seq``.
+
+    Examples
+    ========
+    >>> from sympy.utilities.iterables import has_variety
+    >>> from sympy import Dict, Set
+
+    >>> has_variety((1, 2, 1))
+    True
+    >>> has_variety((1, 1, 1))
+    False
+    """
+    for i, s in enumerate(seq):
+        if i == 0:
+            sentinel = s
+        else:
+            if s != sentinel:
+                return True
+    return False
+
 def uniq(seq):
-    '''
+    """
     Remove repeated elements from an iterable, preserving order of first
     appearance.
 
     Returns a sequence of the same type of the input, or a list if the input
     was not a sequence.
 
-    Examples:
-    --------
+    Examples
+    ========
+
     >>> from sympy.utilities.iterables import uniq
     >>> uniq([1,4,1,5,4,2,1,2])
     [1, 4, 5, 2]
@@ -1007,7 +1124,7 @@ def uniq(seq):
     >>> uniq(x for x in (1,4,1,5,4,2,1,2))
     [1, 4, 5, 2]
 
-    '''
+    """
     from sympy.core.function import Tuple
     seen = set()
     result = (s for s in seq if not (s in seen or seen.add(s)))
@@ -1026,9 +1143,11 @@ def generate_bell(n):
 
     Reference:
     [1] Generating involutions, derangements, and relatives by ECO
-        Vincent Vajnovszki, DMTCS vol 1 issue 12, 2010
+    Vincent Vajnovszki, DMTCS vol 1 issue 12, 2010
+
+    Examples
+    ========
 
-    Examples:
     >>> from sympy.utilities.iterables import generate_bell
     >>> list(generate_bell(3))
     [(0, 1, 2), (0, 2, 1), (1, 0, 2), (2, 0, 1), (2, 1, 0)]
@@ -1069,9 +1188,10 @@ def generate_involutions(n):
     Reference:
     http://mathworld.wolfram.com/PermutationInvolution.html
 
-    Examples:
-    >>> from sympy.utilities.iterables import \
-    generate_involutions
+    Examples
+    ========
+
+    >>> from sympy.utilities.iterables import generate_involutions
     >>> generate_involutions(3)
     [(0, 1, 2), (0, 2, 1), (1, 0, 2), (2, 1, 0)]
     >>> len(generate_involutions(4))
@@ -1108,7 +1228,9 @@ def generate_derangements(perm):
     ECO operator approach once the permutations
     branch is in master.
 
-    Examples:
+    Examples
+    ========
+
     >>> from sympy.utilities.iterables import generate_derangements
     >>> list(generate_derangements([0,1,2]))
     [[1, 2, 0], [2, 0, 1]]
@@ -1138,7 +1260,9 @@ def unrestricted_necklace(n, k):
     Reference:
     http://mathworld.wolfram.com/Necklace.html
 
-    Examples:
+    Examples
+    ========
+
     >>> from sympy.utilities.iterables import unrestricted_necklace
     >>> [i[:] for i in unrestricted_necklace(3, 2)]
     [[0, 0, 0], [0, 1, 1]]
@@ -1174,10 +1298,11 @@ def generate_oriented_forest(n):
     Reference:
     [1] T. Beyer and S.M. Hedetniemi: constant time generation of \
         rooted trees, SIAM J. Computing Vol. 9, No. 4, November 1980
-    [2] http://stackoverflow.com/questions/1633833/
-        oriented-forest-taocp-algorithm-in-python
+    [2] http://stackoverflow.com/questions/1633833/oriented-forest-taocp-algorithm-in-python
+
+    Examples
+    ========
 
-    Examples:
     >>> from sympy.utilities.iterables import generate_oriented_forest
     >>> list(generate_oriented_forest(4))
     [[0, 1, 2, 3], [0, 1, 2, 2], [0, 1, 2, 1], [0, 1, 2, 0], \
@@ -1201,3 +1326,145 @@ def generate_oriented_forest(n):
                     break
             else:
                 break
+
+def lazyDSU_sort(seq, keys, warn=False):
+    """Return sorted seq, breaking ties by applying keys only when needed.
+
+    If ``warn`` is True then an error will be raised if there were no
+    keys remaining to break ties.
+
+    Examples
+    ========
+
+    >>> from sympy.utilities.iterables import lazyDSU_sort, default_sort_key
+    >>> from sympy.abc import x, y
+
+    The count_ops is not sufficient to break ties in this list and the first
+    two items appear in their original order: Python sorting is stable.
+
+    >>> lazyDSU_sort([y + 2, x + 2, x**2 + y + 3],
+    ...    [lambda x: x.count_ops()], warn=False)
+    ...
+    [y + 2, x + 2, x**2 + y + 3]
+
+    The use of default_sort_key allows the tie to be broken (and warn can
+    be safely left False).
+
+    >>> lazyDSU_sort([y + 2, x + 2, x**2 + y + 3],
+    ...    [lambda x: x.count_ops(),
+    ...     default_sort_key])
+    ...
+    [x + 2, y + 2, x**2 + y + 3]
+
+    Here, sequences are sorted by length, then sum:
+
+    >>> seq, keys = [[[1, 2, 1], [0, 3, 1], [1, 1, 3], [2], [1]], [
+    ...    lambda x: len(x),
+    ...    lambda x: sum(x)]]
+    ...
+    >>> lazyDSU_sort(seq, keys, warn=False)
+    [[1], [2], [1, 2, 1], [0, 3, 1], [1, 1, 3]]
+
+    If ``warn`` is True, an error will be raised if there were not
+    enough keys to break ties:
+
+    >>> lazyDSU_sort(seq, keys, warn=True)
+    Traceback (most recent call last):
+    ...
+    ValueError: not enough keys to break ties
+
+
+    Notes
+    =====
+
+    The decorated sort is one of the fastest ways to sort a sequence for
+    which special item comparison is desired: the sequence is decorated,
+    sorted on the basis of the decoration (e.g. making all letters lower
+    case) and then undecorated. If one wants to break ties for items that
+    have the same decorated value, a second key can be used. But if the
+    second key is expensive to compute then it is inefficient to decorate
+    all items with both keys: only those items having identical first key
+    values need to be decorated. This function applies keys successively
+    only when needed to break ties.
+    """
+    d = defaultdict(list)
+    keys = list(keys)
+    f = keys.pop(0)
+    for a in seq:
+        d[f(a)].append(a)
+    seq = []
+    for k in sorted(d.keys()):
+        if len(d[k]) > 1:
+            if keys:
+                d[k] = lazyDSU_sort(d[k], keys, warn)
+            elif warn:
+              raise ValueError('not enough keys to break ties')
+            seq.extend(d[k])
+        else:
+            seq.append(d[k][0])
+    return seq
+
+def minlex(seq, directed=True):
+    """Return a tuple where the smallest element appears first; if
+    ``directed`` is True (default) then the order is preserved, otherwise the
+    sequence will be reversed if that gives a lexically smaller ordering.
+
+    Examples
+    ========
+    >>> from sympy.combinatorics.polyhedron import minlex
+    >>> minlex((1, 2, 0))
+    (0, 1, 2)
+    >>> minlex((1, 0, 2))
+    (0, 2, 1)
+    >>> minlex((1, 0, 2), directed=False)
+    (0, 1, 2)
+    """
+    seq = list(seq)
+    small = min(seq)
+    i = seq.index(small)
+    if not directed:
+        n = len(seq)
+        p = (i + 1) % n
+        m = (i - 1) % n
+        if seq[p] > seq[m]:
+            seq = list(reversed(seq))
+            i = n - i - 1
+    if i:
+        seq = rotate_left(seq, i)
+    return tuple(seq)
+
+def runs(seq, op=gt):
+    """Group the sequence into lists in which successive elements
+    all compare the same with the comparison operator, ``op``:
+    op(seq[i + 1], seq[i]) is True from all elements in a run.
+
+    Examples
+    ========
+
+    >>> from sympy.utilities.iterables import runs
+    >>> from operator import ge
+    >>> runs([0, 1, 2, 2, 1, 4, 3, 2, 2])
+    [[0, 1, 2], [2], [1, 4], [3], [2], [2]]
+    >>> runs([0, 1, 2, 2, 1, 4, 3, 2, 2], op=ge)
+    [[0, 1, 2, 2], [1, 4], [3], [2, 2]]
+    """
+    cycles = []
+    seq = iter(seq)
+    try:
+        run = [seq.next()]
+    except StopIteration:
+        return []
+    while True:
+        try:
+            ei = seq.next()
+        except StopIteration:
+            break
+        if op(ei, run[-1]):
+            run.append(ei)
+            continue
+        else:
+            cycles.append(run)
+            run = [ei]
+    if run:
+        cycles.append(run)
+    return cycles
